@@ -94,37 +94,6 @@ set interfaces {{ type }} {{ name }} peer {{ peer.name }} public-key '{{ peer.pu
 {% endif -%}
 """
 
-vyos_bgp_peer = """
-{%for peer in peers -%}
-{%if peer.state == "absent" -%}
-delete protocols bgp {{ localAS }} neighbor {{ peer.ip }}
-{% else -%}
-{%if peer.state == "disabled"-%}
-set protocols bgp {{ localAS }} neighbor {{ peer.ip }} shutdown
-{%endif-%}
-
-{% if peer.state == "present" -%}
-delete protocols bgp {{ localAS }} neighbor {{ peer.ip }} disable
-{% endif -%}
-
-set protocols bgp {{ localAS }} neighbor {{ peer.ip }} remote-as {{ peer.remote_as }}
-{%if peer.desc is defined and peer.desc|length -%}
-set protocols bgp {{ localAS }} neighbor {{ peer.ip }} description {{ peer.desc }}
-{% endif -%}
-{%if peer.ebgp_multihop is defined and peer.ebgp_multihop|length -%}
-set protocols bgp {{ localAS }} neighbor {{ peer.ip }} ebgp-multihop {{ peer.ebgp_multihop }}
-{% endif -%}
-{%if peer.route_maps -%}
-{%for rm in peer.route_maps -%}  
-{%if rm.state == "absent" -%}
-delete protocols bgp {{ localAS }} neighbor {{ peer.ip }} address-family ipv4-unicast route-map {{ rm.action }} {{ rm.route_map }}
-{% else -%}
-set protocols bgp {{ localAS }} neighbor {{ peer.ip }} address-family ipv4-unicast route-map {{ rm.action }} {{ rm.route_map }}
-{%endif -%}
-{%endfor -%}
-{%endif -%} 
-{%endif -%}
-{% endfor -%}"""
 
 
 vyos_ospf = """
@@ -167,16 +136,58 @@ set protocols ospf area {{ net.area }} network {{ net.subnet }}{{ net.mask }}
 
 {% endfor -%}"""                                   
 
+vyos_bgp_asn = """
+set protocols bgp local-as {{ bgp_asn }}
+"""
 
-vyos_bgp_prefixes = """
-{% for prefix in prefixes -%}
-{% if prefix.state == "absent" -%}
-delete protocols bgp {{ localAS }} address-family {{ prefix.address_family }} network {{ prefix.prefix }}{{ prefix.mask }}
+vyos_bgp_prefix = """
+{% if state == "absent" -%}
+delete protocols bgp address-family {{ address_family }} network {{ prefix }}{{ mask }}
 {% else -%}
-set protocols bgp {{ localAS }} address-family {{ prefix.address_family }} network {{ prefix.prefix }}{{ prefix.mask }}
+set protocols bgp address-family {{ address_family }} network {{ prefix }}{{ mask }}
 {% endif -%}
-{% endfor -%}"""
+"""
 
+vyos_bgp_peer = """
+{%if state == "absent" -%}
+delete protocols bgp neighbor {{ peer_ip }}
+
+{% else -%}
+
+{% if state == "shutdown" -%}
+set protocols bgp neighbor {{ peer_ip }} shutdown
+{% endif -%}
+
+{% if state == "present" -%}
+delete protocols bgp neighbor {{ peer_ip }} shutdown
+{% endif -%}
+
+set protocols bgp neighbor {{ peer_ip }} remote-as {{ remote_as }}
+
+{%if desc is defined and desc|length -%}
+set protocols bgp neighbor {{ peer_ip }} description {{ desc }}
+{% endif -%}
+
+{%if ebgp_multihop is defined and ebgp_multihop|length -%}
+set protocols bgp neighbor {{ peer_ip }} ebgp-multihop {{ ebgp_multihop }}
+{% endif -%}
+
+{%if route_maps -%}
+{%for rm in route_maps -%}  
+
+{%if rm.state == "absent" -%}
+
+delete protocols bgp neighbor {{ peer_ip }} address-family ipv4-unicast route-map {{ rm.action }} {{ rm.route_map }}
+
+{% else -%}
+set protocols bgp neighbor {{ peer_ip }} address-family ipv4-unicast route-map {{ rm.action }} {{ rm.route_map }}
+{% endif -%}
+{% endfor -%}
+
+{%endif -%} 
+{%endif -%}
+
+"""
 
 vyos_routemap = """
 {%- for map in route_maps -%}
@@ -258,30 +269,50 @@ set firewall name {{ ruleset.name }} rule {{ rule.rule_no }} description '{{ rul
 
 {% if rule.dest is defined -%}
 
-{% if rule.dest["address"] is defined and rule.dest["address"]|length -%}
-set firewall name {{ ruleset.name }} rule {{ rule.rule_no }} destination address "{{ rule.dest["address"] }}"
+{% for d in rule.dest -%}
+
+{% if d["address"] is defined and d["address"]|length -%}
+set firewall name {{ ruleset.name }} rule {{ rule.rule_no }} destination address "{{ d['address'] }}"
 {% endif -%}
 
-{% if rule.dest["port"] is defined and rule.dest["port"]|length -%}
-set firewall name {{ ruleset.name }} rule {{ rule.rule_no }} destination port "{{ rule.dest["port"] }}"
+{% if d['port'] is defined and d['port']|length -%}
+set firewall name {{ ruleset.name }} rule {{ rule.rule_no }} destination port "{{ d['port'] }}"
 {% endif -%}
 
-{% if rule.dest["group"] is defined and rule.dest["group"]|length -%}
-set firewall name {{ ruleset.name }} rule {{ rule.rule_no }} destination group {{ rule.dest["group"] }}
+{% if d['group'] is defined and d['group']|length -%}
+set firewall name {{ ruleset.name }} rule {{ rule.rule_no }} destination group {{ d['group'] }}
 {% endif -%}
 
+{% endfor-%}
 
 {% endif -%}
 
 
 {% if rule.source is defined and rule.source|length -%}
-set firewall name {{ ruleset.name }} rule {{ rule.rule_no }} source '{{ rule.source }}' 
+
+{% for s in rule.source -%}
+
+{% if s["address"] is defined and s["address"]|length -%}
+set firewall name {{ ruleset.name }} rule {{ rule.rule_no }} source address "{{ s['address'] }}"
+{% endif -%}
+
+{% if s["port"] is defined and s["port"]|length -%}
+set firewall name {{ ruleset.name }} rule {{ rule.rule_no }} source port "{{ s['port'] }}"
+{% endif -%}
+
+{% if s['group'] is defined and s['group']|length -%}
+set firewall name {{ ruleset.name }} rule {{ rule.rule_no }} source group {{ s['group'] }}
+{% endif -%}
+
+{% endfor-%}
+
 {%endif -%}
 
 {% if rule.protocol is defined and rule.protocol|length -%}
 set firewall name {{ ruleset.name }} rule {{ rule.rule_no }} protocol '{{ rule.protocol }}' 
 {%endif -%}
 
+
 {% if rule.states is defined -%}
 
 {% for state in rule.states -%}
@@ -299,6 +330,14 @@ set firewall name {{ ruleset.name }} rule {{ rule.rule_no }} state {{ state.name
 
 {%endfor -%}"""
 
+vyos_groups="""
+set firewall group {{type}}-group {{ name }}
+set firewall group {{type}}-group {{ name }} description '{{ desc }}'
+{% for net in networks -%}
+set firewall group {{type}}-group {{ name }} network {{ net }}
+{% endfor -%}
+"""
+
 vyos_zones = """
 {% for zone in zones -%}
 
