@@ -1,7 +1,7 @@
 from jinja2 import Template
 from netmiko import Netmiko, ConnectHandler
-# from net_automation import j2templates
-import j2templates
+from net_automation import j2templates
+# import j2templates
 import datetime
 import os
 import yaml
